<?php
/**
 * Plugin Name: WP Immunity
 * Plugin URI:  https://wpimmunity.com
 * Description: WordPress Immunity / Security, and Protection 
 * Version:     0.6
 * Author:      fuzzylogic
 * Text Domain: wpimmune-text
 * Author URI:  https://wpimmunity.com
 * License URI: https://wpimmunity.com/tos
 */

if (!defined('ABSPATH')) {
    die('Error 404');
}

const WPIM_CSP = 'upgrade-insecure-requests;';

function wpim_flush_rewrite_rules(): void {
    global $wp_rewrite;
}
function wpim_get_filesystem(): ?WP_Filesystem_Base {

    global $wp_filesystem;

    return $wp_filesystem;
}

function wpim_cleanup_htaccess(): void {
    $filesystem = wpim_get_filesystem();
    if ( null === $filesystem ) {
        return;
    }

    $htaccess_file = wpim_plugin_get_htaccess( $filesystem );
    if ( null === $htaccess_file ) {
        return;
    }

    $htaccess_contents = $filesystem->get_contents( $htaccess_file );
    if ( false === $htaccess_contents ) {
        return;
    }

    // This regex should match our whole block, both current and older versions.
    $regex             = '/\n*# (BEGIN )?(WordPress )?WPIM - XSS Protection?\d+\.\d+\.\d+[^#]+\n# END (WordPress )?WPIM - XSS Protection[^\n]*\n*/ism';
    $htaccess_contents = preg_replace( $regex, "\n\n", $htaccess_contents );
    $filesystem->put_contents( $htaccess_file, $htaccess_contents );

    wpim_flush_rewrite_rules();
}

function wpim_plugin_get_htaccess(WP_Filesystem_Base $filesystem ): ?string {
    $htaccess_file = get_home_path() . '.htaccess';
    if ( ! $filesystem->exists( $htaccess_file ) || ! $filesystem->is_readable( $htaccess_file ) || ! $filesystem->is_writable( $htaccess_file ) ) {
        return null;
    }
    return $htaccess_file;
}
function wpim_get_csp_header(): string {
    $csp = get_option('wpim_csp');
    return empty($csp) ? WPIM_CSP : $csp;
}
function wpim_update_htaccess(): void {
    $filesystem = wpim_get_filesystem();
    if ( null === $filesystem ) {
        return;
    }

    $htaccess_file = wpim_plugin_get_htaccess($filesystem);
    if ( null === $htaccess_file ) {
        return;
    }

    wpim_cleanup_htaccess();

    $original_contents = $filesystem->get_contents($htaccess_file);
    if ( false === $original_contents ) {
        return;
    }

    $lines = array(
        $original_contents,
        '',
        sprintf( '# BEGIN WPIM - XSS Protection'),
        '<IfModule mod_headers.c>',
    );

    foreach (wpim_get_headers() as $name => $header) {
        $lines[] = "Header set {$name} \"{$header}\"";
    }

    $lines[] = '</IfModule>';
    $lines[] = '# END WPIM - XSS Protection';
    $lines[] = '';

    $filesystem->put_contents($htaccess_file, implode( PHP_EOL, $lines ));
}

function wpim_get_hsts_header(): string {
    $max_age            = get_option( 'wpim_max_age' );
    $include_subdomains = get_option( 'wpim_include_subdomains' );
    $preload            = get_option( 'wpim_preload' );

    $header_tokens = array( "max-age={$max_age}" );
    if ( $include_subdomains ) {
        $header_tokens[] = 'includeSubDomains';
    }
    if ( $preload ) {
        $header_tokens[] = 'preload';
    }

    return implode( '; ', $header_tokens );
}

function wpim_get_headers( array $headers = array() ): array {
    $headers['Access-Control-Allow-Methods']             = 'GET,POST';
    $headers['Access-Control-Allow-Headers']             = 'Content-Type, Authorization';
    $headers['Content-Security-Policy']                  = wpim_get_csp_header();
    $headers['Cross-Origin-Embedder-Policy']             = "unsafe-none; report-to='default'";
    $headers['Cross-Origin-Embedder-Policy-Report-Only'] = "unsafe-none; report-to='default'";
    $headers['Cross-Origin-Opener-Policy']               = 'unsafe-none';
    $headers['Cross-Origin-Opener-Policy-Report-Only']   = "unsafe-none; report-to='default'";
    $headers['Cross-Origin-Resource-Policy']             = 'cross-origin';
    $headers['Permissions-Policy']                       = 'interest-cohort=(), window-management=(), accelerometer=(), autoplay=(), camera=(), cross-origin-isolated=(), display-capture=(self), encrypted-media=(), fullscreen=*, geolocation=(self), gyroscope=(), keyboard-map=(), magnetometer=(), microphone=(), midi=(), payment=*, picture-in-picture=(), publickey-credentials-get=(), screen-wake-lock=(), sync-xhr=(), usb=(), xr-spatial-tracking=(), gamepad=(), serial=()';
    $headers['Referrer-Policy']                          = 'strict-origin-when-cross-origin';
    $headers['Strict-Transport-Security']                = wpim_get_hsts_header();
    $headers['X-Content-Security-Policy']                = 'default-src \'self\'; img-src *; media-src * data:;';
    $headers['X-Content-Type-Options']                   = 'nosniff';
    // $headers['X-Frame-Options']                          = 'SAMEORIGIN';
    $headers['X-XSS-Protection']                         = '1; mode=block';
    $headers['X-Permitted-Cross-Domain-Policies']        = 'none';

    return $headers;
}
add_filter( 'wp_headers', 'wpim_get_headers' );

//define('wpim_PLUGIN_PATH', plugin_dir_path(__FILE__));
add_filter( 'xmlrpc_enabled', '__return_false' );
//Check if is in the Admin Panel
if (is_admin()) {
    $URL_Link = 'https://wpimmunity.com';
    //WP Immunity Admin Menus
    add_action('admin_menu', 'wpim_adminmenu');
    
    function wpim_adminmenu()
    {
        $page_title = 'Dashboard';
        $menu_title = 'WP Immunity';
        $capability = 'manage_options';
        $menu_slug  = 'wpimmunity';
        $function   = 'wpim_dashboard';
        $icon_url   = 'dashicons-shield-alt';
        
        add_menu_page($page_title, $menu_title, $capability, $menu_slug, $function, $icon_url);

        add_submenu_page('wpimmunity', 'Dashboard', 'Dashboard', 'manage_options', 'wpimmunity-dashboard', 'wpim_dashboard');
        add_submenu_page('wpimmunity', 'Pricing', 'Pricing', 'manage_options', 'wpimmunity-pricing', 'wpim_pricing');
        add_submenu_page('wpimmunity', 'Logs', 'Logs', 'manage_options', 'wpimmunity-logs', 'wpim_logs');
        add_submenu_page(null, 'Login History', 'Login History', 'manage_options', 'wpimmunity-dashboard', 'wpim_dashboard');
        add_submenu_page(null, 'System Information', 'System Information', 'manage_options', 'wpimmunity-info', 'wpim_info');
        add_submenu_page(null, 'Analytics', 'Analytics', 'manage_options', 'wpimmunity-live', 'wpim_live');
        add_submenu_page(null, '.htaccess Editor', '.htaccess Editor', 'manage_options', 'wpimmunity-htaccess', 'wpim_htaccess');
        add_submenu_page(null, 'Settings', 'Settings', 'manage_options', 'wpimmunity-settings', 'wpim_settings');
        add_submenu_page(null, 'Bans', 'Bans', 'manage_options', 'wpimmunity-bans', 'wpim_bans');

    }
    
    //WP Immunity Action Links
    add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'wpim_actionlinks');
    //Remove possible user enumeration REST API
    add_filter( 'rest_endpoints', function( $endpoints ){
        if ( isset( $endpoints['/wp/v2/users'] ) ) {
            unset( $endpoints['/wp/v2/users'] );
        }    return $endpoints;
    });
    function wpim_actionlinks($links)
    {
        $links[] = '<a href="' . esc_url(get_admin_url(null, 'admin.php?page=wpimmunity-settings')) . '">Settings</a>';
        $links[] = '<a href="https://wpimmunity.com" target="_blank">Support</a>';
        return $links;
    }
    
    //WP Immunity Database Options
    add_option('wpim_mail_notifications', '0', '', 'yes');
    add_option('wpim_countryban_blacklist', '1', '', 'yes');
    add_option('wpim_sqli_protection', '1', '', 'yes');
    add_option('wpim_sqli_protection2', '1', '', 'yes');
    add_option('wpim_sqli_protection3', '0', '', 'yes');
    add_option('wpim_sqli_protection4', '1', '', 'yes');
    add_option('wpim_sqli_protection5', '1', '', 'yes');
    add_option('wpim_sqli_protection6', '1', '', 'yes');
    add_option('wpim_sqli_logging', '1', '', 'yes');
    add_option('wpim_sqli_autoban', '0', '', 'yes');
    add_option('wpim_sqli_mail', '0', '', 'yes');
    add_option('wpim_sqli_redirect', get_site_url() . '/blocked-request', '', 'yes');
    add_option('wpim_badbot_protection', '1', '', 'yes');
    add_option('wpim_badbot_protection2', '1', '', 'yes');
    add_option('wpim_badbot_protection3', '1', '', 'yes');
    add_option('wpim_badbot_logging', '1', '', 'yes');
    add_option('wpim_badbot_autoban', '0', '', 'yes');
    add_option('wpim_badbot_mail', '0', '', 'yes');
    add_option('wpim_proxy_protection', '0', '', 'yes');
    add_option('wpim_proxy_protection2', '0', '', 'yes');
	add_option('wpim_proxy_api1', '', '', 'yes');
	add_option('wpim_proxy_api2', '', '', 'yes');
	add_option('wpim_proxy_api3', '', '', 'yes');
    add_option('wpim_proxy_logging', '1', '', 'yes');
    add_option('wpim_proxy_mail', '0', '', 'yes');
    add_option('wpim_proxy_redirect', get_site_url() . '/proxy-detected', '', 'yes');
    add_option('wpim_spam_protection', '0', '', 'yes');
    add_option('wpim_spam_logging', '1', '', 'yes');
    add_option('wpim_spam_mail', '0', '', 'yes');
    add_option('wpim_spam_redirect', get_site_url() . '/spam-ip', '', 'yes');
    add_option('wpim_live_traffic', '0', '', 'yes');
    add_option('wpim_error_reporting', '5', '', 'yes');
    add_option('wpim_display_errors', '0', '', 'yes');
    add_option('wpim_banned_redirect', get_site_url() . '/banned', '', 'yes');
    add_option('wpim_bannedc_redirect', get_site_url() . '/banned-country', '', 'yes');
    add_option('wpim_bannedo_redirect', get_site_url() . '/banned-os', '', 'yes');
    add_option('wpim_bannedb_redirect', get_site_url() . '/banned-browser', '', 'yes');
    add_option('wpim_bannedi_redirect', get_site_url() . '/banned-isp', '', 'yes');
    add_option('wpim_bannedr_redirect', get_site_url() . '/banned-referer', '', 'yes');
    add_option('wpim_max_age', '63072000');
    add_option('wpim_include_subdomains', '0');
    add_option('wpim_preload', '0');
    add_option('wpim_csp', WPIM_CSP);

    add_option('wpim_license', '');
    add_option('wpim_license_expire', '');
    add_option('wpim_license_plan', '');
    
    wpim_update_htaccess();
    //Warning Pages
    function wpim_addwpage()
    {
        $wppage1_content = '<p>You are banned and you cannot continue to the website</p>
		<p><br /><br /></p>
		<p>Protected by <strong><a href="https://wpimmunity.com" target="_blank">WP Immunity</strong></p>
		';
		$wpage1  = array(
            'post_title' => 'Banned',
            'post_content' => $wppage1_content,
            'post_status' => 'publish',
            'post_author' => 1,
            'post_type' => 'page'
        );
		
		$wppage2_content = '<p>Malicious request was detected</p>
		<p><br /><br /></p>
		<p>Protected by <strong><a href="https://wpimmunity.com" target="_blank">WP Immunity</strong></p>
		';
        $wpage2  = array(
            'post_title' => 'Blocked Request',
            'post_content' => $wppage2_content,
            'post_status' => 'publish',
            'post_author' => 1,
            'post_type' => 'page'
        );
		
		$wppage3_content = '<p>Access to the website via Proxy is not allowed (Disable Browser Data Compression if you have it enabled)</p>
		<p><br /><br /></p>
		<p>Protected by <strong><a href="https://wpimmunity.com" target="_blank">WP Immunity</strong></p>
		';
        $wpage3  = array(
            'post_title' => 'Proxy Detected',
            'post_content' => $wppage3_content,
            'post_status' => 'publish',
            'post_author' => 1,
            'post_type' => 'page'
        );
		
		$wppage4_content = '<p>You are in the Blacklist of Spammers and you cannot continue to the website</p>
		<p><br /><br /></p>
		<p>Protected by <strong><a href="https://wpimmunity.com" target="_blank">WP Immunity</strong></p>
		';
        $wpage4  = array(
            'post_title' => 'Spam IP',
            'post_content' => $wppage4_content,
            'post_status' => 'publish',
            'post_author' => 1,
            'post_type' => 'page'
        );
		
		$wppage5_content = '<p>Sorry, but your country is banned and you cannot continue to the website</p>
		<p><br /><br /></p>
		<p>Protected by <strong><a href="https://wpimmunity.com" target="_blank">WP Immunity</strong></p>
		';
        $wpage5  = array(
            'post_title' => 'Banned Country',
            'post_content' => $wppage5_content,
            'post_status' => 'publish',
            'post_author' => 1,
            'post_type' => 'page'
        );
		
		$wppage6_content = '<p>Access to the website through your Browser is not allowed, please use another Internet Browser</p>
		<p><br /><br /></p>
		<p>Protected by <strong><a href="https://wpimmunity.com" target="_blank">WP Immunity</strong></p>
		';
        $wpage6  = array(
            'post_title' => 'Banned Browser',
            'post_content' => $wppage6_content,
            'post_status' => 'publish',
            'post_author' => 1,
            'post_type' => 'page'
        );
		
		$wppage7_content = '<p>Access to the website through your Operating System is not allowed</p>
		<p><br /><br /></p>
		<p>Protected by <strong><a href="https://wpimmunity.com" target="_blank">WP Immunity</strong></p>
		';
        $wpage7  = array(
            'post_title' => 'Banned OS',
            'post_content' => $wppage7_content,
            'post_status' => 'publish',
            'post_author' => 1,
            'post_type' => 'page'
        );
		
		$wppage8_content = '<p>Your Internet Service Provider is blacklisted and you cannot continue to the website</p>
		<p><br /><br /></p>
		<p>Protected by <strong><a href="https://wpimmunity.com" target="_blank">WP Immunity</strong></p>
		';
        $wpage8  = array(
            'post_title' => 'Banned ISP',
            'post_content' => $wppage8_content,
            'post_status' => 'publish',
            'post_author' => 1,
            'post_type' => 'page'
        );
		
		$wppage9_content = '<p>Your referrer url is blocked and you cannot continue to the website</p>
		<p><br /><br /></p>
		<p>Protected by <strong><a href="https://wpimmunity.com" target="_blank">WP Immunity</strong></p>
		';
        $wpage9  = array(
            'post_title' => 'Banned Referer',
            'post_content' => $wppage9_content,
            'post_status' => 'publish',
            'post_author' => 1,
            'post_type' => 'page'
        );
        
        if (get_page_by_title('Banned') == NULL) {
            wp_insert_post($wpage1);
        }
        if (get_page_by_title('Blocked Request') == NULL) {
            wp_insert_post($wpage2);
        }
        if (get_page_by_title('Proxy Detected') == NULL) {
            wp_insert_post($wpage3);
        }
        if (get_page_by_title('Spam IP') == NULL) {
            wp_insert_post($wpage4);
        }
        if (get_page_by_title('Banned Country') == NULL) {
            wp_insert_post($wpage5);
        }
        if (get_page_by_title('Banned Browser') == NULL) {
            wp_insert_post($wpage6);
        }
        if (get_page_by_title('Banned OS') == NULL) {
            wp_insert_post($wpage7);
        }
        if (get_page_by_title('Banned ISP') == NULL) {
            wp_insert_post($wpage8);
        }
        if (get_page_by_title('Banned Referer') == NULL) {
            wp_insert_post($wpage9);
        }
    }
    register_activation_hook(__FILE__, 'wpim_addwpage');
    
    //WP Immunity Database Tables
    register_activation_hook(__FILE__, 'wpim_install');
    function wpim_install()
    {
        global $wpdb;
        
        $table_name      = $wpdb->prefix . "wpim_logs";
        $charset_collate = $wpdb->get_charset_collate();
        
        $sql = "CREATE TABLE IF NOT EXISTS $table_name (
  id int(11) NOT NULL AUTO_INCREMENT,
  ip char(45) NOT NULL,
  date varchar(50) NOT NULL,
  time char(10) NOT NULL,
  page varchar(255) NOT NULL,
  query text NOT NULL,
  type varchar(50) NOT NULL,
  browser varchar(255) DEFAULT 'Unknown' NOT NULL,
  browser_code varchar(50) NOT NULL,
  os varchar(255) DEFAULT 'Unknown' NOT NULL,
  os_code varchar(40) NOT NULL,
  country varchar(120) DEFAULT 'Unknown' NULL,
  country_code char(2) DEFAULT 'XX' NULL,
  region varchar(120) DEFAULT 'Unknown' NULL,
  city varchar(120) DEFAULT 'Unknown' NULL,
  latitude varchar(30) DEFAULT '0' NULL,
  longitude varchar(30) DEFAULT '0' NULL,
  isp varchar(255) DEFAULT 'Unknown' NULL,
  useragent text NOT NULL,
  referer_url varchar(255) NULL,
  PRIMARY KEY (id)
) $charset_collate;";
        
        $table_name2 = $wpdb->prefix . "wpim_bans";
        $sql2        = "CREATE TABLE IF NOT EXISTS $table_name2 (
  id int(11) NOT NULL AUTO_INCREMENT,
  type varchar(50) NOT NULL,
  value varchar(255) NOT NULL,
  date varchar(50) NULL,
  time char(10) NULL,
  reason varchar(255) NULL,
  autoban tinyint(1) DEFAULT '0' NULL,
  PRIMARY KEY (id)
) $charset_collate;";
        
        $table_name3 = $wpdb->prefix . "wpim_ipwhitelist";
        $sql3        = "CREATE TABLE IF NOT EXISTS $table_name3 (
  id int(11) NOT NULL AUTO_INCREMENT,
  ip varchar(45) NOT NULL,
  notes varchar(255) NULL,
  PRIMARY KEY (id)
) $charset_collate;";
        
        $table_name4 = $wpdb->prefix . "wpim_dnsbl";
        $sql4        = "CREATE TABLE IF NOT EXISTS $table_name4 (
  id int(11) NOT NULL AUTO_INCREMENT,
  dnsbl_database varchar(30) NOT NULL,
  PRIMARY KEY (id)
) $charset_collate;";
        
        $table_name5 = $wpdb->prefix . "wpim_livetraffic";
        $sql5        = "CREATE TABLE IF NOT EXISTS $table_name5 (
  id int(11) NOT NULL AUTO_INCREMENT,
  ip char(45) NOT NULL,
  useragent varchar(255) NULL,
  browser varchar(255) DEFAULT 'Unknown' NOT NULL,
  browser_code varchar(50) NOT NULL,
  os varchar(255) DEFAULT 'Unknown' NOT NULL,
  os_code varchar(40) NOT NULL,
  device_type varchar(12) NOT NULL,
  country varchar(120) DEFAULT 'Unknown' NULL,
  country_code char(2) DEFAULT 'XX' NULL,
  request_uri varchar(255) NOT NULL,
  referer varchar(255) NULL,
  bot tinyint(1) DEFAULT '0' NOT NULL,
  date varchar(50) NOT NULL,
  time char(10) NOT NULL,
  uniquev tinyint(1) DEFAULT '0' NOT NULL,
  PRIMARY KEY (id)
) $charset_collate;";

        $table_name6 = $wpdb->prefix . "wpim_loginhistory";
        $sql6        = "CREATE TABLE IF NOT EXISTS $table_name6 (
  id int(11) NOT NULL AUTO_INCREMENT,
  username varchar(255) NOT NULL,
  ip char(45) NULL,
  date varchar(30) NULL,
  time char(5) NULL,
  successful int(1) NULL,
  PRIMARY KEY (id)
) $charset_collate;";

		$table_name7 = $wpdb->prefix . "wpim_filewhitelist";
        $sql7        = "CREATE TABLE IF NOT EXISTS $table_name7 (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `path` char(255) NOT NULL,
  `notes` varchar(255) NOT NULL,
   PRIMARY KEY (id)
) $charset_collate;";
        
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
        dbDelta($sql2);
        dbDelta($sql3);
        dbDelta($sql4);
        dbDelta($sql5);
        dbDelta($sql6);
		dbDelta($sql7);
        
        $dnsblc1 = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM $table_name4
                    WHERE dnsbl_database = %s LIMIT 1", 'sbl.spamhaus.org'));
        if ($dnsblc1 <= 0) {
            $wpdb->insert($table_name4, array(
                'dnsbl_database' => 'sbl.spamhaus.org'
            ), array(
                '%s'
            ));
        }
        
        $dnsblc1 = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM $table_name4
                    WHERE dnsbl_database = %s LIMIT 1", 'xbl.spamhaus.org'));
        if ($dnsblc1 <= 0) {
            $wpdb->insert($table_name4, array(
                'dnsbl_database' => 'xbl.spamhaus.org'
            ), array(
                '%s'
            ));
        }
    }
	
    // CSS Styles
    function wpim_styles()
    {
        if (isset($_GET['page']) && strpos($_GET['page'], 'wpguard') === 0) {
            wp_enqueue_style('bootstrap460c', 'https://cdn.jsdelivr.net/npm/bootstrap@4.6.2/dist/css/bootstrap.min.css');
            wp_enqueue_style('fontawesome', 'https://use.fontawesome.com/releases/v5.15.4/css/all.css');
            wp_enqueue_style('admincss', 'https://cdnjs.cloudflare.com/ajax/libs/admin-lte/3.2.0/css/adminlte.min.css');
            if (sanitize_text_field($_GET['page']) == 'wpguard-bans') {
                wp_enqueue_style('select2c', 'https://cdn.jsdelivr.net/npm/select2@4.0.13/dist/css/select2.min.css');
            }
            wp_enqueue_style('swticheryc', 'https://cdnjs.cloudflare.com/ajax/libs/switchery/0.8.2/switchery.min.css');
            wp_enqueue_style('datatablesc', 'https://cdn.datatables.net/v/bs4/dt-1.13.1/r-2.4.0/datatables.min.css');
            wp_enqueue_style('flags', esc_url(plugins_url('assets/plugins/flags/flags.css',__FILE__)));
            $wpgcustom_css = "
            body {
				background: #f1f1f1;
			}
			h1.protmodg {
				color:#47A447;
			}
			h1.protmodr {
				color:#d2322d;
			}
			h1.protmodb {
				color:#007bff;
			}
			h4.dashboardlb {
				background-color:#f7f7f7; 
				font-size: 16px; 
				text-align: center; 
				padding: 7px 10px; 
				margin-top: 0;
			}
			.wpanelbg {
				background-color:#23282d;
			}";
            wp_register_style('wpguard-css', false);
            wp_enqueue_style('wpguard-css');
            wp_add_inline_style('wpguard-css', $wpgcustom_css);
        }
    }
    add_action('admin_enqueue_scripts', 'wpim_styles');
    
    // JS Scripts
    function wpim_scripts()
    {
        if (isset($_GET['page']) && strpos($_GET['page'], 'wpguard') === 0) {
            wp_enqueue_script('swtichery', 'https://cdnjs.cloudflare.com/ajax/libs/switchery/0.8.2/switchery.min.js');
            wp_enqueue_script('jquery361', 'https://code.jquery.com/jquery-3.6.1.min.js');
            if (sanitize_text_field($_GET['page']) == 'wpguard' || sanitize_text_field($_GET['page']) == 'wpguard-analytics') {
                wp_enqueue_script('chartjs', 'https://cdnjs.cloudflare.com/ajax/libs/Chart.js/3.9.1/chart.min.js');
            }
            if (sanitize_text_field($_GET['page']) == 'wpguard-logdetails') {
                wp_enqueue_script('openlayers', 'https://openlayers.org/api/OpenLayers.js');
            }
			if (sanitize_text_field($_GET['page']) == 'wpguard-modules') {
                wp_enqueue_script('popperjs', 'https://cdn.jsdelivr.net/npm/popper.js@1.16.1/dist/umd/popper.min.js');
            }
            wp_enqueue_script('bootstrap460', 'https://cdn.jsdelivr.net/npm/bootstrap@4.6.2/dist/js/bootstrap.min.js');
            wp_enqueue_script('adminjs', 'https://cdnjs.cloudflare.com/ajax/libs/admin-lte/3.2.0/js/adminlte.min.js');
            if (sanitize_text_field($_GET['page']) == 'wpguard-bans') {
                wp_enqueue_script('select2', 'https://cdn.jsdelivr.net/npm/select2@4.0.13/dist/js/select2.min.js');
            }
            wp_enqueue_script('datatables', 'https://cdn.datatables.net/v/bs4/dt-1.13.1/r-2.4.0/datatables.min.js');
            
            $wpgcustom_js = '
        $(document).ready(function() {

	$(\'#dt-basic\').dataTable( {
		"responsive": true,
        "order": [[ 0, "desc" ]],
		"language": {
			"paginate": {
			  "previous": \'<i class="fas fa-angle-left"></i>\',
			  "next": \'<i class="fas fa-angle-right"></i>\'
			}
		}
	} );
	$(\'#dt-basic2\').dataTable( {
		"responsive": true,
        "order": [[ 0, "desc" ]],
		"language": {
			"paginate": {
			  "previous": \'<i class="fas fa-angle-left"></i>\',
			  "next": \'<i class="fas fa-angle-right"></i>\'
			}
		}
	} );
	$(\'#dt-basiclt\').dataTable( {
		"responsive": true,
        "order": [[ 2, "asc" ]],
		"language": {
			"paginate": {
			  "previous": \'<i class="fas fa-angle-left"></i>\',
			  "next": \'<i class="fas fa-angle-right"></i>\'
			}
		}
	} );
	$(\'#dt-basiclt1\').dataTable( {
		"responsive": true,
        "order": [[ 5, "desc" ]],
		"language": {
			"paginate": {
			  "previous": \'<i class="fas fa-angle-left"></i>\',
			  "next": \'<i class="fas fa-angle-right"></i>\'
			}
		}
	} );
	$(\'#dt-basicdb\').dataTable( {
		"responsive": true,
        "order": [[ 1, "desc" ]],
		"language": {
			"paginate": {
			  "previous": \'<i class="fas fa-angle-left"></i>\',
			  "next": \'<i class="fas fa-angle-right"></i>\'
			}
		}
	} );
} );

var elems = Array.prototype.slice.call(document.querySelectorAll(\'.psec-switch\'));

elems.forEach(function(html) {
  var switchery = new Switchery(html, {secondaryColor: \'red\'});
});
';
			
            wp_register_script('wpguard-js', '', [], '', true);
            wp_enqueue_script('wpguard-js');
            wp_add_inline_script('wpguard-js', $wpgcustom_js);
			
			if (sanitize_text_field($_GET['page']) == 'wpguard-bans') {
				$wpgcustom_js2 = '
$(document).ready(function() {
	$(\'#dt-basic3\').dataTable( {
		"responsive": true,
        "order": [[ 0, "desc" ]],
		"language": {
			"paginate": {
			  "previous": \'<i class="fas fa-angle-left"></i>\',
			  "next": \'<i class="fas fa-angle-right"></i>\'
			}
		}
	} );
	$(\'#dt-basic4\').dataTable( {
		"responsive": true,
        "order": [[ 0, "desc" ]],
		"language": {
			"paginate": {
			  "previous": \'<i class="fas fa-angle-left"></i>\',
			  "next": \'<i class="fas fa-angle-right"></i>\'
			}
		}
	} );
	$(\'#dt-basic5\').dataTable( {
		"responsive": true,
        "order": [[ 0, "desc" ]],
		"language": {
			"paginate": {
			  "previous": \'<i class="fas fa-angle-left"></i>\',
			  "next": \'<i class="fas fa-angle-right"></i>\'
			}
		}
	} );
	$(\'#dt-basic6\').dataTable( {
		"responsive": true,
        "order": [[ 0, "desc" ]],
		"language": {
			"paginate": {
			  "previous": \'<i class="fas fa-angle-left"></i>\',
			  "next": \'<i class="fas fa-angle-right"></i>\'
			}
		}
	} );
	$(\'#dt-basic7\').dataTable( {
		"responsive": true,
        "order": [[ 0, "desc" ]],
		"language": {
			"paginate": {
			  "previous": \'<i class="fas fa-angle-left"></i>\',
			  "next": \'<i class="fas fa-angle-right"></i>\'
			}
		}
	} );
} );

$(".select2").select2({
    width: \'100%\'
});
				';
				wp_register_script('wpguard-js2', '', [], '', true);
				wp_enqueue_script('wpguard-js2');
				wp_add_inline_script('wpguard-js2', $wpgcustom_js2);
			}
			
			if (sanitize_text_field($_GET['page']) == 'wpguard-seccheck') {
				$wpgcustom_js2 = '
					$("table").addClass("table table-bordered table-hover");
				';
				wp_register_script('wpguard-js2', '', [], '', true);
				wp_enqueue_script('wpguard-js2');
				wp_add_inline_script('wpguard-js2', $wpgcustom_js2);
			}
        }
    }
    add_action('admin_enqueue_scripts', 'wpim_scripts');
	
    //WP Immunity Admin Pages
    function wpim_header()
    {
?>
<div class="wrap">
<?php
    }
    
    
    function wpim_logs()
    {
        global $wpdb;
        wpim_header();
        
        $pagetitle = "Logs";
        include_once "wpim_header.php";
        include_once "wpim_logs.php";
        echo '</div>';
    }

    function wpim_dashboard()
    {
        global $wpdb;
        wpim_header();
        
        $pagetitle = "Dashboard";
        include_once "wpim_header.php";
        include_once "wpim_dashboard.php";
        echo '</div>';
    }
    function wpim_info()
    {
        global $wpdb;
        wpim_header();
        
        $pagetitle = "System Information";
        include_once "wpim_header.php";
        include_once "wpim_info.php";
        echo '</div>';
    }
    function wpim_pricing()
    {
        global $wpdb;
        wpim_header();
        
        $pagetitle = "Pricing";
        include_once "wpim_header.php";
        include_once "wpim_pricing.php";
        echo '</div>';
    }
    function wpim_bans()
    {
        global $wpdb;
        wpim_header();
        
        $pagetitle = "Bans";
        include_once "wpim_header.php";
        include_once "wpim_bans.php";
        echo '</div>';
    }
    function wpim_htaccess()
    {
        global $wpdb;
        wpim_header();
        
        $pagetitle = ".htaccess Editor";
        include_once "wpim_header.php";
        include_once "wpim_htaccess.php";
        echo '</div>';
    }
    function wpim_settings()
    {
        global $wpdb;
        wpim_header();
        
        $pagetitle = "Settings";
        include_once "wpim_header.php";
        include_once "wpim_settings.php";
        echo '</div>';
    }
    function wpim_live()
    {
        global $wpdb;
        wpim_header();
        
        $pagetitle = "Live Statistics";
        include_once "wpim_header.php";
        include_once "wpim_live.php";
        echo '</div>';
    }

    
    //Uninstall
    register_uninstall_hook(__FILE__, 'wpim_uninstall');
    
    function wpim_uninstall()
    {
        global $wpdb;
        
        include_once "uninstall.php";
    }
    
} else {
    //WP Immunity Initialization
    add_action('init', 'wpim_include');
    wpim_update_htaccess();
    function wpim_include()
    {
        global $wpdb;
        
        include(plugin_dir_path(__FILE__) . 'modules/core.php');
        
        //IP Whitelist
        $table           = $wpdb->prefix . 'wpim_ipwhitelist';
        $whitelist_check = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM $table
                    WHERE ip = %s LIMIT 1", $ip));
		$tablef           = $wpdb->prefix . 'wpim_filewhitelist';
        $fwhitelist_check = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM $tablef
                    WHERE path = %s LIMIT 1", $script_name));
        
        if ($whitelist_check <= 0 && $fwhitelist_check <= 0) {
            
            if (get_option('wpim_error_reporting') == 1) {
                error_reporting(0);
            }
            if (get_option('wpim_error_reporting') == 2) {
                error_reporting(E_ERROR | E_WARNING | E_PARSE);
            }
            if (get_option('wpim_error_reporting') == 3) {
                error_reporting(E_ERROR | E_WARNING | E_PARSE | E_NOTICE);
            }
            if (get_option('wpim_error_reporting') == 4) {
                error_reporting(E_ALL & ~E_NOTICE);
            }
            if (get_option('wpim_error_reporting') == 5) {
                error_reporting(E_ALL);
            }
            
            //Displaying Errors
            if (get_option('wpim_display_errors') == 1) {
                ini_set('display_errors', '1');
            } else {
                ini_set('display_errors', '0');
            }
            
            include(plugin_dir_path(__FILE__) . 'modules/ban-system.php');
			include(plugin_dir_path(__FILE__) . 'modules/sqli-protection.php');
			if ($actual_url != get_option('wpim_banned_redirect') && $actual_url != (get_option('wpim_banned_redirect') . '/')) {
				include(plugin_dir_path(__FILE__) . 'modules/badbots-protection.php');
				include(plugin_dir_path(__FILE__) . 'modules/fakebots-protection.php');
				include(plugin_dir_path(__FILE__) . 'modules/headers-check.php');
				if ($searchengine_bot == 0) {
					include(plugin_dir_path(__FILE__) . 'modules/proxy-protection.php');
					include(plugin_dir_path(__FILE__) . 'modules/spam-protection.php');
				}
			}
			include(plugin_dir_path(__FILE__) . 'modules/live-traffic.php');
        }
    }
}

//Login History
add_action('wp_login', 'wpim_saveslogin', 10, 1);
add_action('wp_login_failed', 'wpim_saveflogin', 10, 1);

$ip = $_SERVER['REMOTE_ADDR'];
if ($ip == "::1") {
	$ip = "127.0.0.1";
}
$date   = date_i18n(get_option('date_format'));
$time   = date_i18n(get_option('time_format'));

function wpim_saveslogin($user_login) {
	
    global $wpdb;
    global $ip, $date, $time;
	
	$table  = $wpdb->prefix . 'wpim_loginhistory';
	$data   = array(
            'username' => $user_login,
            'ip' => $ip,
            'date' => $date,
            'time' => $time,
            'successful' => '1'
        );
        $format = array(
            '%s',
            '%s',
            '%s',
            '%s',
            '%s',
            '%d'
        );
        $wpdb->insert($table, $data, $format);
}

function wpim_saveflogin($user_login) {
	
    global $wpdb;
    global $ip, $date, $time;
	
	$table  = $wpdb->prefix . 'wpim_loginhistory';
	$data   = array(
            'username' => $user_login,
            'ip' => $ip,
            'date' => $date,
            'time' => $time,
            'successful' => '0'
        );
        $format = array(
            '%s',
            '%s',
            '%s',
            '%s',
            '%s',
            '%d'
        );
        $wpdb->insert($table, $data, $format);
}

?>